
/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/* --- Message-based communication for module icu --------------- */

#if defined(__RTAI__) && defined(__KERNEL__)
# include <linux/kernel.h>
#else
# include <stdio.h>
# include <stddef.h>
#endif

#include <portLib.h>
#include <errnoLib.h>

#include "icuMsgLib.h"
#include "icuError.h"
 
/*========================================================================
 *
 *  icuClientInit  -  Routine d'initialisation d'un client de icu
 *
 *  Description:
 *  Cette routine  doit etre appelee par un client du module, pour pouvoir 
 *  l'utiliser. Attention: la fonction csMboxInit doit etre appelee avant.
 *
 *  Retourne: OK ou ERROR
 */
 
STATUS icuClientInit (CLIENT_ID *pClientId)

{
  STATUS status;

  /* Appeler la routine d'initialisation d'un client CS */
  status = csClientInit (ICU_MBOX_NAME, ICU_MAX_RQST_SIZE,
			 ICU_MAX_INTERMED_REPLY_SIZE, 
			 ICU_MAX_REPLY_SIZE, 
			pClientId);
  if (status != OK) 
    h2perror("icuClientInit");

  icuRecordH2errMsgs();

  return status;
}
 
/*-------------------------------------------------------------------------
 *
 *  icuClientEnd - Routine de fin d'un client de icu
 *
 *  Description:
 *  Cette routine libere les objets alloues par le client.
 *
 *  Retourne : OK ou ERROR
 */

STATUS icuClientEnd (CLIENT_ID clientId)         /* Id. du client */
     
{
  STATUS status;
  /* Appeler la routine de liberation du client */
  status = csClientEnd (clientId);
  if (status != OK) 
    h2perror("icuClientEnd");
  return status;
}


/*========================================================================
 *
 *  icuAbortRqstSend  -  Emission d'une requete de controle
 *
 *  Description: Arret de l'activite specifie'e en parametre
 *
 *  Retourne : OK ou ERROR
 */
 
int icuAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ABORT_RQST, (void *) activity,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return(ERROR);

  return(OK);
}

/*-------------------------------------------------------------------------
 *
 *  icuAbortReplyRcv  -  Reception de la replique finale
 *
 *  Description:
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
			     (FUNCPTR) NULL, (void *) NULL,
			     0, (FUNCPTR) NULL);
  if (status == ERROR) {
    *bilan = errnoGet();

    /* Filtrage type d'erreur */
    if (H2_MODULE_ERR_FLAG(*bilan))
      return(FINAL_REPLY_OK);
  }
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *
 *  icuAbortRqstAndRcv  -  Requete de controle
 *
 *  Description: Arret de l'activite specifie'e en parametre
 *  Cette fonction emet une requete et attend la replique finale.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan)

{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ABORT_RQST, (void *) activity,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			     (void *) NULL, 0, (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  if (status == ERROR) {
    *bilan = errnoGet();
    
    /* Filtrage type d'erreur */
    if (H2_MODULE_ERR_FLAG(*bilan))
      return(FINAL_REPLY_OK);
  }
  
  return(status);
}


/* liste des requetes/repliques de controle : msgCntrl.c */

/* liste des requetes/repliques d'execution : msgExec.c */





/*========================================================================
 *
 *  icuInitDisplayRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuInitDisplayRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_AFFICHAGE *(in_display),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_INITDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuInitDisplayReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuInitDisplayReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuInitDisplayRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuInitDisplayRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_AFFICHAGE *(in_display), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_INITDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuInitDisplayRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuInitDisplayRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_AFFICHAGE *(in_display), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_INITDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 

/*=========================================================================
 *
 *  icuPomTaggingRqstSend  -  Emission d'une requete de controle
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuPomTaggingRqstSend (CLIENT_ID clientId, 
				  int *pRqstId,
				  int *(in_tag_on_off),
				  int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_POMTAGGING_RQST, (void *) in_tag_on_off,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuPomTaggingReplyRcv  -  Reception de la replique finale
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuPomTaggingReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuPomTaggingRqstAndRcv  -  Emission/reception requete de controle
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuPomTaggingRqstAndRcv (CLIENT_ID clientId, 
				 int *(in_tag_on_off), 
				 
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_POMTAGGING_RQST, (void *) in_tag_on_off,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, &rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
				  (void *) NULL, 0, (FUNCPTR) NULL, 
				  (void *) NULL, 0, 
				  (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}



/*=========================================================================
 *
 *  icuAutoModeRqstSend  -  Emission d'une requete de controle
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuAutoModeRqstSend (CLIENT_ID clientId, 
				  int *pRqstId,
				  ICU_AUTOSWITCH_CONTROL *(in_autoSwitchControl),
				  int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_AUTOMODE_RQST, (void *) in_autoSwitchControl,
			sizeof(ICU_AUTOSWITCH_CONTROL), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuAutoModeReplyRcv  -  Reception de la replique finale
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuAutoModeReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuAutoModeRqstAndRcv  -  Emission/reception requete de controle
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuAutoModeRqstAndRcv (CLIENT_ID clientId, 
				 ICU_AUTOSWITCH_CONTROL *(in_autoSwitchControl), 
				 
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_AUTOMODE_RQST, (void *) in_autoSwitchControl,
			sizeof(ICU_AUTOSWITCH_CONTROL), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, &rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
				  (void *) NULL, 0, (FUNCPTR) NULL, 
				  (void *) NULL, 0, 
				  (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}



/*=========================================================================
 *
 *  icuSaveFaceRqstSend  -  Emission d'une requete de controle
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuSaveFaceRqstSend (CLIENT_ID clientId, 
				  int *pRqstId,
				  int *(in_save_face),
				  int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SAVEFACE_RQST, (void *) in_save_face,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuSaveFaceReplyRcv  -  Reception de la replique finale
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuSaveFaceReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuSaveFaceRqstAndRcv  -  Emission/reception requete de controle
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuSaveFaceRqstAndRcv (CLIENT_ID clientId, 
				 int *(in_save_face), 
				 
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SAVEFACE_RQST, (void *) in_save_face,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, &rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
				  (void *) NULL, 0, (FUNCPTR) NULL, 
				  (void *) NULL, 0, 
				  (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}




/*========================================================================
 *
 *  icuCaptureImageRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuCaptureImageRqstSend (CLIENT_ID clientId, int *pRqstId,
				int *(in_captureImage),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CAPTUREIMAGE_RQST, (void *) in_captureImage,
			sizeof(int), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuCaptureImageReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuCaptureImageReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuCaptureImageRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuCaptureImageRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 int *(in_captureImage), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CAPTUREIMAGE_RQST, (void *) in_captureImage,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuCaptureImageRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuCaptureImageRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 int *(in_captureImage), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CAPTUREIMAGE_RQST, (void *) in_captureImage,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 

/*=========================================================================
 *
 *  icuCaptureVideoRqstSend  -  Emission d'une requete de controle
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuCaptureVideoRqstSend (CLIENT_ID clientId, 
				  int *pRqstId,
				  int *(in_captureVideo),
				  int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CAPTUREVIDEO_RQST, (void *) in_captureVideo,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuCaptureVideoReplyRcv  -  Reception de la replique finale
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuCaptureVideoReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuCaptureVideoRqstAndRcv  -  Emission/reception requete de controle
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuCaptureVideoRqstAndRcv (CLIENT_ID clientId, 
				 int *(in_captureVideo), 
				 
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CAPTUREVIDEO_RQST, (void *) in_captureVideo,
			sizeof(int), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, &rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
				  (void *) NULL, 0, (FUNCPTR) NULL, 
				  (void *) NULL, 0, 
				  (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}



/*=========================================================================
 *
 *  icuSetViamPosterNameRqstSend  -  Emission d'une requete de controle
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuSetViamPosterNameRqstSend (CLIENT_ID clientId, 
				  int *pRqstId,
				  str *(in_viamPosterName),
				  int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SETVIAMPOSTERNAME_RQST, (void *) in_viamPosterName,
			sizeof(str), (FUNCPTR) NULL, FALSE, 0, replyTimeOut, 
			pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuSetViamPosterNameReplyRcv  -  Reception de la replique finale
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou
 *             WAITING_FINAL_REPLY
 */
 
int icuSetViamPosterNameReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, (void *) NULL, 0,  
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuSetViamPosterNameRqstAndRcv  -  Emission/reception requete de controle
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK
 */
 
int icuSetViamPosterNameRqstAndRcv (CLIENT_ID clientId, 
				 str *(in_viamPosterName), 
				 
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SETVIAMPOSTERNAME_RQST, (void *) in_viamPosterName,
			sizeof(str), (FUNCPTR) NULL, FALSE, 0, 
			TIME_WAIT_REPLY, &rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique */
  if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
				  (void *) NULL, 0, (FUNCPTR) NULL, 
				  (void *) NULL, 0, 
				  (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}




/*========================================================================
 *
 *  icuEnableDisplayRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuEnableDisplayRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_AFFICHAGE *(in_display),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ENABLEDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuEnableDisplayReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuEnableDisplayReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuEnableDisplayRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuEnableDisplayRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_AFFICHAGE *(in_display), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ENABLEDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuEnableDisplayRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuEnableDisplayRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_AFFICHAGE *(in_display), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ENABLEDISPLAY_RQST, (void *) in_display,
			sizeof(ICU_AFFICHAGE), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuSetCameraPosRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuSetCameraPosRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_CAMERA_POS *(in_cameraPos),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SETCAMERAPOS_RQST, (void *) in_cameraPos,
			sizeof(ICU_CAMERA_POS), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuSetCameraPosReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuSetCameraPosReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuSetCameraPosRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuSetCameraPosRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_CAMERA_POS *(in_cameraPos), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SETCAMERAPOS_RQST, (void *) in_cameraPos,
			sizeof(ICU_CAMERA_POS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuSetCameraPosRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuSetCameraPosRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_CAMERA_POS *(in_cameraPos), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_SETCAMERAPOS_RQST, (void *) in_cameraPos,
			sizeof(ICU_CAMERA_POS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuTrackItRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuTrackItRqstSend (CLIENT_ID clientId, int *pRqstId,
				int *(in_track_it),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_TRACKIT_RQST, (void *) in_track_it,
			sizeof(int), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuTrackItReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuTrackItReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuTrackItRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuTrackItRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 int *(in_track_it), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_TRACKIT_RQST, (void *) in_track_it,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuTrackItRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuTrackItRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 int *(in_track_it), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_TRACKIT_RQST, (void *) in_track_it,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForSomebodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForSomebodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				int *(in_nbImageMaxNobody),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForSomebodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForSomebodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       ICU_WAITFOR *(out_waitFor),
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) out_waitFor,
				  sizeof(ICU_WAITFOR), (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForSomebodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForSomebodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 ICU_WAITFOR *(out_waitFor),
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) out_waitFor, sizeof(ICU_WAITFOR), 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForSomebodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForSomebodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 ICU_WAITFOR *(out_waitFor),
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) out_waitFor, sizeof(ICU_WAITFOR), 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) out_waitFor, sizeof(ICU_WAITFOR), 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForSomeoneRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForSomeoneRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_WAITFOR *(in_waitFor),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForSomeoneReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForSomeoneReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForSomeoneRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForSomeoneRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_WAITFOR *(in_waitFor), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForSomeoneRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForSomeoneRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_WAITFOR *(in_waitFor), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORSOMEONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForNobodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForNobodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				int *(in_nbImageMaxNobody),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForNobodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForNobodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForNobodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForNobodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForNobodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForNobodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODY_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForNoneRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForNoneRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_WAITFOR *(in_waitFor),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForNoneReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForNoneReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForNoneRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForNoneRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_WAITFOR *(in_waitFor), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForNoneRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForNoneRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_WAITFOR *(in_waitFor), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNONE_RQST, (void *) in_waitFor,
			sizeof(ICU_WAITFOR), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForNobodyKnownRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForNobodyKnownRqstSend (CLIENT_ID clientId, int *pRqstId,
				int *(in_nbImageMaxNobody),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODYKNOWN_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForNobodyKnownReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForNobodyKnownReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForNobodyKnownRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForNobodyKnownRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODYKNOWN_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForNobodyKnownRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForNobodyKnownRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 int *(in_nbImageMaxNobody), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORNOBODYKNOWN_RQST, (void *) in_nbImageMaxNobody,
			sizeof(int), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForPersonListChangeRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForPersonListChangeRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_PERSON_LIST_PARAMS *(in_paramsPersonList),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORPERSONLISTCHANGE_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForPersonListChangeReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForPersonListChangeReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       ICU_WAITFOR *(out_waitFor),
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) out_waitFor,
				  sizeof(ICU_WAITFOR), (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForPersonListChangeRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForPersonListChangeRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_PERSON_LIST_PARAMS *(in_paramsPersonList), 
				 ICU_WAITFOR *(out_waitFor),
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORPERSONLISTCHANGE_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) out_waitFor, sizeof(ICU_WAITFOR), 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForPersonListChangeRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForPersonListChangeRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_PERSON_LIST_PARAMS *(in_paramsPersonList), 
				 ICU_WAITFOR *(out_waitFor),
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORPERSONLISTCHANGE_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) out_waitFor, sizeof(ICU_WAITFOR), 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) out_waitFor, sizeof(ICU_WAITFOR), 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuProcessSetModalityRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuProcessSetModalityRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_MODALITIES *(in_setModality),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_PROCESSSETMODALITY_RQST, (void *) in_setModality,
			sizeof(ICU_MODALITIES), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuProcessSetModalityReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuProcessSetModalityReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuProcessSetModalityRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuProcessSetModalityRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_MODALITIES *(in_setModality), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_PROCESSSETMODALITY_RQST, (void *) in_setModality,
			sizeof(ICU_MODALITIES), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuProcessSetModalityRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuProcessSetModalityRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_MODALITIES *(in_setModality), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_PROCESSSETMODALITY_RQST, (void *) in_setModality,
			sizeof(ICU_MODALITIES), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecSetWorkingPathRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecSetWorkingPathRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_facerecWorkingPath),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETWORKINGPATH_RQST, (void *) in_facerecWorkingPath,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetWorkingPathReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecSetWorkingPathReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecSetWorkingPathRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecSetWorkingPathRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_facerecWorkingPath), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETWORKINGPATH_RQST, (void *) in_facerecWorkingPath,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetWorkingPathRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecSetWorkingPathRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_facerecWorkingPath), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETWORKINGPATH_RQST, (void *) in_facerecWorkingPath,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecSetParamsRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecSetParamsRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_PERSON_LIST_PARAMS *(in_paramsPersonList),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPARAMS_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetParamsReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecSetParamsReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecSetParamsRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecSetParamsRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_PERSON_LIST_PARAMS *(in_paramsPersonList), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPARAMS_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetParamsRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecSetParamsRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_PERSON_LIST_PARAMS *(in_paramsPersonList), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPARAMS_RQST, (void *) in_paramsPersonList,
			sizeof(ICU_PERSON_LIST_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecInitRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecInitRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECINIT_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecInitReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecInitReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecInitRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECINIT_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecInitRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecInitRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECINIT_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecAddSomebodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecAddSomebodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_recoAddName),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECADDSOMEBODY_RQST, (void *) in_recoAddName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecAddSomebodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecAddSomebodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecAddSomebodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecAddSomebodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoAddName), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECADDSOMEBODY_RQST, (void *) in_recoAddName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecAddSomebodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecAddSomebodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoAddName), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECADDSOMEBODY_RQST, (void *) in_recoAddName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecSetPeopleRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecSetPeopleRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_recoName),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPEOPLE_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetPeopleReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecSetPeopleReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecSetPeopleRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecSetPeopleRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPEOPLE_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecSetPeopleRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecSetPeopleRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECSETPEOPLE_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecRecognizeAllRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecRecognizeAllRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECRECOGNIZEALL_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecRecognizeAllReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecRecognizeAllReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecRecognizeAllRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecRecognizeAllRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECRECOGNIZEALL_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecRecognizeAllRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecRecognizeAllRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECRECOGNIZEALL_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecLoadSomebodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecLoadSomebodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_recoName),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecLoadSomebodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecLoadSomebodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecLoadSomebodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecLoadSomebodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecLoadSomebodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecLoadSomebodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecUnloadSomebodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecUnloadSomebodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_recoName),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecUnloadSomebodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecUnloadSomebodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecUnloadSomebodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecUnloadSomebodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecUnloadSomebodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecUnloadSomebodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoName), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADSOMEBODY_RQST, (void *) in_recoName,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecLoadEverybodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecLoadEverybodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecLoadEverybodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecLoadEverybodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecLoadEverybodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecLoadEverybodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecLoadEverybodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecLoadEverybodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuFacerecUnloadEverybodyRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuFacerecUnloadEverybodyRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuFacerecUnloadEverybodyReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuFacerecUnloadEverybodyReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuFacerecUnloadEverybodyRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuFacerecUnloadEverybodyRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuFacerecUnloadEverybodyRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuFacerecUnloadEverybodyRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_FACERECUNLOADEVERYBODY_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuWaitForModalityChangeRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuWaitForModalityChangeRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORMODALITYCHANGE_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuWaitForModalityChangeReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuWaitForModalityChangeReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuWaitForModalityChangeRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuWaitForModalityChangeRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORMODALITYCHANGE_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuWaitForModalityChangeRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuWaitForModalityChangeRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_WAITFORMODALITYCHANGE_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuLaunchPlatineRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuLaunchPlatineRqstSend (CLIENT_ID clientId, int *pRqstId,
				ICU_PLATINE_PARAMS *(in_platineParams),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_LAUNCHPLATINE_RQST, (void *) in_platineParams,
			sizeof(ICU_PLATINE_PARAMS), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuLaunchPlatineReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuLaunchPlatineReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuLaunchPlatineRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuLaunchPlatineRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 ICU_PLATINE_PARAMS *(in_platineParams), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_LAUNCHPLATINE_RQST, (void *) in_platineParams,
			sizeof(ICU_PLATINE_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuLaunchPlatineRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuLaunchPlatineRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 ICU_PLATINE_PARAMS *(in_platineParams), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_LAUNCHPLATINE_RQST, (void *) in_platineParams,
			sizeof(ICU_PLATINE_PARAMS), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuCmdPtuRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuCmdPtuRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CMDPTU_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuCmdPtuReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuCmdPtuReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuCmdPtuRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuCmdPtuRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CMDPTU_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuCmdPtuRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuCmdPtuRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_CMDPTU_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuIcuInitRFIDRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuIcuInitRFIDRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUINITRFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuIcuInitRFIDReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuIcuInitRFIDReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuIcuInitRFIDRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuIcuInitRFIDRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUINITRFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuIcuInitRFIDRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuIcuInitRFIDRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUINITRFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuIcuCloseRFIDRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuIcuCloseRFIDRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUCLOSERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuIcuCloseRFIDReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuIcuCloseRFIDReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuIcuCloseRFIDRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuIcuCloseRFIDRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUCLOSERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuIcuCloseRFIDRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuIcuCloseRFIDRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUCLOSERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuIcuAcquireRFIDRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuIcuAcquireRFIDRqstSend (CLIENT_ID clientId, int *pRqstId,
				
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuIcuAcquireRFIDReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuIcuAcquireRFIDReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuIcuAcquireRFIDRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuIcuAcquireRFIDRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				  
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuIcuAcquireRFIDRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuIcuAcquireRFIDRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				  
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRERFID_RQST, (void *) NULL,
			0, (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 


/*========================================================================
 *
 *  icuIcuAcquireTagRqstSend  -  Emission d'une requete d'execution
 *
 *  Retourne : OK ou ERROR
 */
 
STATUS icuIcuAcquireTagRqstSend (CLIENT_ID clientId, int *pRqstId,
				GEN_STRING256 *(in_recoTag),
				int replyTimeOut)
{
  errnoSet(0);

  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRETAG_RQST, (void *) in_recoTag,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, TIME_WAIT_REPLY, 
			replyTimeOut, pRqstId) == ERROR)
    return ERROR;
  return OK;
}

/*-------------------------------------------------------------------------
 *  icuIcuAcquireTagReplyRcv  -  Reception des repliques
 *
 *  Retourne : ERROR ou FINAL_REPLY_OK ou 
 *             WAITING_INTERMED_REPLY ou INTERMED_REPLY_TIMEOUT ou
 *             WAITING_FINAL_REPLY    ou FINAL_REPLY_TIMEOUT
 */
 
int icuIcuAcquireTagReplyRcv (CLIENT_ID clientId, int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
					    BLOCK_ON_INTERMED_REPLY */
			       
			       int *activity, int *bilan)
{
  int status;    /* ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK */

  /* Verifier le flag de type de blocage */
  if (block != NO_BLOCK && block != BLOCK_ON_FINAL_REPLY 
      && block != BLOCK_ON_INTERMED_REPLY) {
    errnoSet (S_icu_stdGenoM_BAD_BLOCK_TYPE);
    return (ERROR);
  }
  
  if ((status = csClientReplyRcv (clientId, rqstId, block, 
				  (void *) activity, sizeof(int), 
				  (FUNCPTR) NULL, (void *) NULL,
				  0, (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
  
  return(status);
}
 
/*-------------------------------------------------------------------------
 *  icuIcuAcquireTagRqstAndAck  -  Emet la requete d'exec, 
 *                                  attend la replique intermediaire
 *
 *  Retourne : ERROR ou INTERMED_REPLY_TIMEOUT ou WAITING_FINAL_REPLY ou 
 *             FINAL_REPLY_OK 
 */
 
int icuIcuAcquireTagRqstAndAck (CLIENT_ID clientId, int *pRqstId,
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoTag), 
				 
				 int *activity, int *bilan)
{
  int status;
  
  errnoSet(0);
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRETAG_RQST, (void *) in_recoTag,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			pRqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  if ((status = csClientReplyRcv (clientId, *pRqstId, BLOCK_ON_INTERMED_REPLY, 
                             (void *) activity, sizeof(int), (FUNCPTR) NULL, 
                             (void *) NULL, 0, 
                             (FUNCPTR) NULL)) == ERROR) {
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
  }
  else *bilan = OK;
 
  return(status);
}

/*-------------------------------------------------------------------------
 *  icuIcuAcquireTagRqstAndRcv  -  Emet la requete d'execution,
 *                                  Attend les repliques.
 *
 *  Retourne : ERROR ou FINAL_REPLY_TIMEOUT ou FINAL_REPLY_OK ou 
 *             INTERMED_REPLY_TIMEOUT
 */
 
int icuIcuAcquireTagRqstAndRcv (CLIENT_ID clientId, 
				 int replyTimeOut,
				 GEN_STRING256 *(in_recoTag), 
				 
				 int *activity,
				 int *bilan)
{
  int rqstId;                  /* Ou` mettre l'id de la requete */
  int status;
  
  errnoSet(0);
  *bilan = OK;
  
  /* Emettre la requete */
  if (csClientRqstSend (clientId, ICU_ICUACQUIRETAG_RQST, (void *) in_recoTag,
			sizeof(GEN_STRING256), (FUNCPTR) NULL, TRUE, 
			TIME_WAIT_REPLY, replyTimeOut, 
			&rqstId) == ERROR) {
    *bilan = errnoGet();
    return(ERROR);
  }
  
  /* Reception de la replique intermediaire */
  status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_INTERMED_REPLY, 
			     (void *) activity, sizeof(int), (FUNCPTR) NULL, 
			     (void *) NULL, 0, 
			     (FUNCPTR) NULL);
  switch(status) {

  case ERROR:
    *bilan = errnoGet();
    if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    break;

  case WAITING_FINAL_REPLY:
    if ((status = csClientReplyRcv (clientId, rqstId, BLOCK_ON_FINAL_REPLY, 
			       (void *) NULL, 0, (FUNCPTR) NULL, 
			       (void *) NULL, 0, 
			       (FUNCPTR) NULL)) == ERROR) {
      *bilan = errnoGet();
      if (H2_MODULE_ERR_FLAG(*bilan)) return(FINAL_REPLY_OK);
    }
  }
  return(status);
}
 
