/*
 *    SONY EVI D30/D31 Interface Code, Version 1.3
 *    Copyright (C) Vladimir Kravtchenko 1998
 *    All rights reserved
 *
 *
 *    Permission to use, copy, and distribute this software is hereby
 *    granted without fee.
 *
 *    Redistribution and use of this code, with or without
 *    modification, is subject to the following conditions:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *    3. All advertising materials mentioning features or use of this code
 *       must display the following acknowledgement: This product includes
 *       software developed by Vladimir Kravtchenko <vk@cs.ubc.ca>.
 *    4. The name of the author may not be used to endorse or promote products 
 *       derived from this software without specific prior written permission.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 *    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *    DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *    INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *    STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *    POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * The inclosed code provides an interface to a single SONY EVI D30/D31 Pan/Tilt camera
 * (address 1), connected to computer by a RS232 port. It defines a class (EviCamera)
 * and provides execution of all VISCA commands and inquiries through the class
 * member functions. See class definition and VISCA/RS232C Command List for details.
 *
 *   Before compilation !
 *
 * - define one of the PLATFORM_OS constants, SUN_UNIX or PC_LINUX
 * - define SERIAL_DEVICE, the port to which EVI Camera is connected
 * - define DEBUG if you want the TX/RX messages to be displayed
 * 
 *   Send your comments and questions to vk@cs.ubc.ca
 *
 *  Modifed by ANTONIO MARIN 
 *  LAAS - CNRS / MIA - UV
 *  sep/2003
 *
 */

#include "evicamera.h"

static int rs232;
static int prevMsg;
static uch RX_Msg[VISCA_PACKET_MAX];


#ifdef SUN_UNIX

void EviCamera_SplitShort ( uch *src, uch *dest ) 
{
  dest[0] = src[0] >> 4;
  dest[1] = src[0] & 0x0F;
  dest[2] = src[1] >> 4;
  dest[3] = src[1] & 0x0F;
}

void EviCamera_AsmShort ( uch *src, uch *dest ) 
{
  dest[0] = src[0] << 4;
  dest[0] = dest[0] | src[1];
  dest[1] = src[2] << 4;
  dest[1] = dest[1] | src[3];
}

#endif

#ifdef PC_LINUX

void EviCamera_SplitShort ( uch *src, uch *dest ) 
{
  dest[0] = src[1] >> 4;
  dest[1] = src[1] & 0x0F;
  dest[2] = src[0] >> 4;
  dest[3] = src[0] & 0x0F;
}

void EviCamera_AsmShort ( uch *src, uch *dest ) 
{
  dest[1] = src[0] << 4;
  dest[1] = dest[1] | src[1];
  dest[0] = src[2] << 4;
  dest[0] = dest[0] | src[3];
}

#endif

// ----------- display_hex displays Visca messages in a heximal format for debugging -------------

void EviCamera_display_hex( unsigned char *msg, int len )
{
  int i;
  for ( i=0; i < len; i++)
      fprintf(stderr, "%x ", msg[i]);

  fprintf(stderr, "\n");
}

// ----------------- write_msg writes Visca messages to the serial port buffer ------------------

Boolean EviCamera_write_msg(int fd1, unsigned char *msg, unsigned char len )
{
  int     rc, lineStatus, lineChk;
  struct  termios *term_st;

        /* Check receiver is still there via LINE_CHECK_FLAG */
 
  rc = ioctl(fd1, TIOCMGET, &lineChk);

  lineStatus = lineChk & LINE_CHECK_FLAG;

  if ( lineStatus != LINE_CHECK_FLAG )
    {
      fprintf(stderr,"Lost connection, line status: ");
      fprintf(stderr,"%o\n", lineStatus);
      return FALSE;
    }

#ifdef  DEBUG
  fprintf(stderr,"\nTX message: ");
  EviCamera_display_hex(msg, (int) len);
#endif
  rc = write(fd1, msg, (int) len);          // Send Visca message
  

  if (rc == -1)
    {
      fprintf(stderr,"Error writing to link\n");
      perror ("Write link");

      /* Flush the link */

      term_st = (struct termios *) malloc (sizeof(struct termios));
      rc = tcgetattr(fd1,term_st);
      if (rc == -1)
                {
		  fprintf(stderr,"Error getting attr for flush\n");
		  perror ("tcgetattr error");
                }
      else
	{
	  term_st->c_lflag = ( FLUSHO | term_st->c_lflag );
	  rc = tcsetattr(fd1,TCSANOW,term_st);
	  if (rc == -1)
	    {
	      fprintf(stderr,"Error flushing link\n");
	      perror ("tcsetattr error");
	    }
	}
      free (term_st);
      return FALSE;
    }
  
  return TRUE;
}


void EviCamera_WaitReady( int NoCom )
{

  while ( NoCom > 0 )
      {
        EviCamera_read_msg(rs232, RX_Msg, VISCA_PACKET_MAX);

        if (RX_Msg[1] >= 0x60)
	   { NoCom = 0;
             fprintf(stderr,"Command execution error!\n");
	   }
        else NoCom--;
      }

}

void EviCamera_SendCommand( uch *msg )
{
  if (EviCamera_write_msg(rs232, &msg[2], msg[0]))
     EviCamera_WaitReady(msg[1]);
  else
     {
       fprintf(stderr,"Error sending command!\n");
       exit(1);
     } 
}

void EviCamera_SendCommand2( uch *msg )
{  
  if (prevMsg>0) EviCamera_WaitReady(prevMsg);
  //  EviCamera_SendCommand(msg );
  if (!EviCamera_write_msg(rs232, &msg[2], msg[0])){
       fprintf(stderr,"Error sending command2!\n");
       prevMsg=0;
       exit(1);
  } else
  prevMsg=msg[1];
  //  printf("%d \n",msg[6]);
}

// ------------------------- set_attr sets attributes of the serial port ------------------------

Boolean EviCamera_set_attr(int fd)
{
  struct  termios *term_st;
  int     rc;

  if (fd > 0)
     {
       term_st = (struct termios *) malloc (sizeof(struct termios));
       rc = tcgetattr(fd,term_st);

       if (rc == -1)
          {
            fprintf(stderr,"Error getting attr\n");
            perror ("tcgetattr error");
            return FALSE;
          }

       /* Setting mode to Raw */

       term_st->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP
                             |INLCR|IGNCR|ICRNL|IXON);
       term_st->c_oflag &= ~(OPOST|ONLCR);
       term_st->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);

       // set MIN and TIME for non-canonical mode input processing
       // MIN = 0 and TIME > 0 such that reads complete as soon
       // as a character is read, or the timer expires. TIME is
       // units of 0.10 seconds

       term_st->c_cc[VMIN] = 0;
       term_st->c_cc[VTIME] = 10;  /* equals 0.5 second */

       // set speed to 9600 baud, 8 databits, even parity enabled
       // 1 stop bit 1 start bit

       term_st->c_cflag = ( ~B300 & term_st->c_cflag );
       term_st->c_cflag = ( B9600 | term_st->c_cflag );

       rc = tcsetattr(fd,TCSANOW,term_st);
       if (rc == -1)
          {
            fprintf(stderr,"Error setting attr\n");
            perror ("tcsetattr error");
            return FALSE;
          }
       else
          {       
            rc = tcgetattr(fd,term_st);  // Set terminal attributes
          }

       free (term_st);

       return TRUE;

     }

  else return FALSE;

}


// --------- create_link opens a serial port for communication with the Visca camera ------------

Boolean EviCamera_create_link(int *fd1)
{
  Boolean status;

  *fd1 = open(SERIAL_DEVICE, O_RDWR, 0);
  if (*fd1 == -1)
  {
    fprintf(stderr,"Error opening link\n");
    perror ("Open error");
    status = FALSE;
  }
  else
  {
    EviCamera_set_attr(*fd1);
    status = TRUE;
  }
  printf("status %d %d\n",status,*fd1);
  return status;
}

// ------------------------------- close_link closes serial port --------------------------------

void EviCamera_close_link(int *fd1)
{
  close(*fd1);
}


// ------------------------------------- VISCA Inquiries --------------------------------------

uch EviCamera_PowerInq( void )
{
  EviCamera_SendCommand( CamPowerInq );
  return RX_Msg[2];
}

ush EviCamera_ZoomPosInq( void )
{
  ush ZoomPos;

  //EviCamera_SendCommand( CamZoomPosInq );
  EviCamera_SendCommand( CamZoomPosInq );
  EviCamera_AsmShort(&RX_Msg[2], (uch*) &ZoomPos);

  return ZoomPos;
}

uch EviCamera_FocusModeInq( void )
{
  EviCamera_SendCommand( CamFocusModeInq );
  return RX_Msg[2];
}

ush EviCamera_FocusPosInq( void )
{
  ush FocusPos;
  EviCamera_SendCommand( CamFocusPosInq );
  EviCamera_AsmShort(&RX_Msg[2], (uch*) &FocusPos);
  return FocusPos;
}

uch EviCamera_WBModeInq( void )
{
  EviCamera_SendCommand( CamWBModeInq );
  return RX_Msg[2];
}

uch EviCamera_AEModeInq( void )
{
  EviCamera_SendCommand( CamAEModeInq );
  return RX_Msg[2];
}

ush EviCamera_ShutterPosInq( void )
{
  ush ShutterPos;
  EviCamera_SendCommand( CamShutterPosInq );
  EviCamera_AsmShort(&RX_Msg[2], (uch*) &ShutterPos);
  return ShutterPos;
}

ush EviCamera_IrisPosInq( void )
{
  ush IrisPos;
  EviCamera_SendCommand( CamIrisPosInq );
  EviCamera_AsmShort(&RX_Msg[2], (uch*) &IrisPos);
  return IrisPos;
}

ush EviCamera_GainPosInq( void )
{
  ush GainPos;
  EviCamera_SendCommand( CamGainPosInq );
  EviCamera_AsmShort(&RX_Msg[2], (uch*) &GainPos);
  return GainPos;
}

uch EviCamera_BackLightModeInq( void )
{
  EviCamera_SendCommand( CamBackLightModeInq );
  return RX_Msg[2];
}

uch EviCamera_MemoryInq( void )
{
  EviCamera_SendCommand( CamMemoryInq );
  return RX_Msg[2];
}

uch EviCamera_KeyLockInq( void )
{
  EviCamera_SendCommand( CamKeyLockInq );
  return RX_Msg[2];
}

uch EviCamera_IDInq( void )
{
  EviCamera_SendCommand( CamIDInq );
  return ((RX_Msg[2] << 4) | RX_Msg[3]);
}

uch EviCamera_IRReceiveModeInq( void )
{
  EviCamera_SendCommand( CamIRReceiveModeInq );
  return RX_Msg[2];;
}

uch EviCamera_VideoSystemInq( void )
{
  EviCamera_SendCommand( CamVideoSystemInq );
  return RX_Msg[2];;
}

uch EviCamera_WideConLensInq( void )
{
  EviCamera_SendCommand( CamWideConLensInq );
  return RX_Msg[3];;
}

ush EviCamera_PanTiltModeInq( void )
{
  ush Mode;
  EviCamera_SendCommand( CamPanTiltModeInq );
  Mode = (ush) RX_Msg[2];
  return ((Mode << 8) | ((ush) RX_Msg[3]));
}

void EviCamera_PanTiltMaxSpdInq( uch *PanSpd, uch *TiltSpd )
{
  EviCamera_SendCommand( CamPanTiltMaxSpdInq );
  *PanSpd  = RX_Msg[2];
  *TiltSpd = RX_Msg[3];
}

void EviCamera_PanTiltPosInq( ssh *PanPos, ssh *TiltPos )
{

  //  EviCamera_SendCommand2( CamPanTiltPosInq );
  EviCamera_SendCommand( CamPanTiltPosInq );
  EviCamera_AsmShort( &RX_Msg[2], (uch*) PanPos  );

  EviCamera_AsmShort( &RX_Msg[6], (uch*) TiltPos );

}

uch EviCamera_DataScreenInq( void )
{
  EviCamera_SendCommand( CamDataScreenInq );
  return RX_Msg[2];;
}

uch EviCamera_AT_MDModeInq( void )
{
  EviCamera_SendCommand( CamAT_MDModeInq );
  return RX_Msg[2];;
}

ush EviCamera_ATModeInq( void )
{
  ush Mode;
  EviCamera_SendCommand( CamATModeInq );
  Mode = (ush) RX_Msg[2];
  return ((Mode << 8) | ((ush) RX_Msg[3]));
}

uch EviCamera_ATEntryInq( void )
{
  EviCamera_SendCommand( CamATEntryInq );
  return RX_Msg[2];
}

ush EviCamera_MDModeInq( void )
{
  ush Mode;
  EviCamera_SendCommand( CamMDModeInq );
  Mode = (ush) RX_Msg[2];
  return ((Mode << 8) | ((ush) RX_Msg[3]));
}

void EviCamera_ATObjPosInq( uch *X, uch *Y, uch *status )
{
  EviCamera_SendCommand( CamATObjPosInq );
  *X      = RX_Msg[2];
  *Y      = RX_Msg[3];
  *status = RX_Msg[4];
}

void EviCamera_MDObjPosInq( uch *X, uch *Y, uch *status )
{
  EviCamera_SendCommand( CamMDObjPosInq );
  *X      = RX_Msg[2];
  *Y      = RX_Msg[3];
  *status = RX_Msg[4];
}

uch EviCamera_MDYLevelInq( void )
{
  EviCamera_SendCommand( CamMDYLevelInq );
  return RX_Msg[3];;
}

uch EviCamera_MDHueLevelInq( void )
{
  EviCamera_SendCommand( CamMDHueLevelInq );
  return RX_Msg[3];;
}

uch EviCamera_MDSizeInq( void )
{
  EviCamera_SendCommand( CamMDSizeInq );
  return RX_Msg[3];;
}

uch EviCamera_MDDispTimeInq( void )
{
  EviCamera_SendCommand( CamMDDispTimeInq );
  return RX_Msg[3];;
}

uch EviCamera_MDRefModeInq( void )
{
  EviCamera_SendCommand( CamMDRefModeInq );
  return RX_Msg[2];
}

uch EviCamera_MDRefTimeInq( void )
{
  EviCamera_SendCommand( CamMDRefTimeInq );
  return RX_Msg[3];;
}




void EviCamera_InitializeCamera( void )
{
  prevMsg = 0;
  EviCamera_SendCommand( CamIFClearB );
  EviCamera_SendCommand( CamSetAddressB );
}

void EviCamera_CreateRS232Link( void )
{
  if (! EviCamera_create_link(&rs232))
     {
       fprintf(stderr, "Error openning ");
       fprintf(stderr, SERIAL_DEVICE);
       fprintf(stderr, "\n");
       exit(1);
     }
}

void EviCamera_CloseRS232Link( void )
{
  EviCamera_close_link(&rs232);
}


void EviCamera_SendCommandPanTilt( int op, uch panSpd,uch tiltSpd)
{  
  uch mesg[]={ 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x03, 0xFF };  

  switch(op) {
  case 10:
    mesg[8]=0x03;
    mesg[9]=0x02;  //CamPTDDown;
    break;
  case 20:
     mesg[8]=0x03;
     mesg[9]=0x01;//mesg=CamPTDUp;
    break;
  case 100:
    mesg[8]=0x02;
    mesg[9]=0x03;// mesg=CamPTDRight;
    break;
  case 200:
     mesg[8]=0x01;
     mesg[9]=0x03;//mesg=CamPTDLeft;
    break;
  case 110:
     mesg[8]=0x02;
     mesg[9]=0x02;//mesg=CamPTDDownRight;
    break;
  case 210:
     mesg[8]=0x01;
     mesg[9]=0x02;//mesg=CamPTDDownLeft;
    break;
  case 120:
    mesg[8]=0x02;
    mesg[9]=0x01; //mesg=CamPTDUpRight;
    break;
  case 220:
    mesg[8]=0x01;
    mesg[9]=0x01;//mesg=CamPTDUpLeft;
    break;
  default:
     mesg[8]=0x03;
     mesg[9]=0x03;//mesg= CamPTDStop; //NULL;
    break;
  }

  mesg[6]= panSpd;
  mesg[7]= tiltSpd;

  //  printf("speed %d %d %d %d\n",mesg[6],mesg[7],panSpd,tiltSpd);
  EviCamera_SendCommand(mesg);
  
}

void EviCamera_SendCommandZoom( int op, uch speed)
{  
  uch newMsg[]   = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x00, 0xFF };

  if (op==10) {
    newMsg[6] = 0x20 ;
  } else if (op==100) {
    newMsg[6] = 0x30 ;
  }else 
    newMsg[6] = 0x00;

  EviCamera_SendCommand(newMsg);
}

// ------------------------------------- VISCA Commands -----------------------------------------

void EviCamera_PowerOn( void )    { EviCamera_SendCommand( CamPowerOn );  }
void EviCamera_PowerOff( void )   { EviCamera_SendCommand( CamPowerOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ZoomStop( void )   { EviCamera_SendCommand( CamZoomStop ); }

void EviCamera_ZoomTeleS( int wait )  
{
  EviCamera_SendCommand( CamZoomTeleS );
  if (wait) while(EviCamera_ZoomPosInq() != ZOOM_POS_MAX);
}

void EviCamera_ZoomWideS( int wait )
{
  EviCamera_SendCommand( CamZoomWideS );
  if (wait) while(EviCamera_ZoomPosInq() != ZOOM_POS_MIN);
}

void EviCamera_ZoomTeleV( uch speed, int wait )  
{
  CamZoomTeleV[6] = 0x20 | speed;

  EviCamera_SendCommand( CamZoomTeleV );
  if (wait) while(EviCamera_ZoomPosInq() != ZOOM_POS_MAX);
}

void EviCamera_ZoomWideV( uch speed, int wait )  
{
  CamZoomWideV[6] = 0x30 | speed;

  EviCamera_SendCommand( CamZoomWideV );
  if (wait) while(EviCamera_ZoomPosInq() != ZOOM_POS_MIN);
}


void EviCamera_ZoomDirect( ush ZoomPos, int wait )
{
  ZoomPos++;         // Adgust Camera and User scales

  EviCamera_SplitShort( (uch*) &ZoomPos, &CamZoomDirect[6] );
  EviCamera_SendCommand( CamZoomDirect );
  if (wait) while(EviCamera_ZoomPosInq() != ZoomPos-1 );
}

// ----------------------------------------------------------------------------------------------
void EviCamera_FocusAutoOn( void )     { EviCamera_SendCommand( CamFocusAutoOn );     }
void EviCamera_FocusManualOn( void )   { EviCamera_SendCommand( CamFocusManualOn );   }
void EviCamera_FocusAutoManual( void ) { EviCamera_SendCommand( CamFocusAutoManual ); }

void EviCamera_FocusStop( void )   { EviCamera_SendCommand( CamFocusStop ); }

void EviCamera_FocusFar( int wait )  
{
  EviCamera_FocusManualOn();
  EviCamera_SendCommand( CamFocusFar );
  if (wait) while(EviCamera_FocusPosInq() != FOCUS_FAR);
}

void EviCamera_FocusNear( int wait )  
{
  ush Foc;
  EviCamera_FocusManualOn();
  EviCamera_SendCommand( CamFocusNear );
  if (wait)
     do    Foc = EviCamera_FocusPosInq();
     while (! ((Foc == FOCUS_NEAR1) || (Foc == FOCUS_NEAR2)));
}

void EviCamera_FocusDirect( ush FocusPos, int wait )
{
  EviCamera_FocusManualOn();
  EviCamera_SplitShort( (uch*) &FocusPos, &CamFocusDirect[6] );
  EviCamera_SendCommand( CamFocusDirect );
  if (wait) while(EviCamera_FocusPosInq() != FocusPos );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_WBAuto( void )     { EviCamera_SendCommand( CamWBAuto );     }
void EviCamera_WBIndoor( void )   { EviCamera_SendCommand( CamWBIndoor );   }
void EviCamera_WBOutdoor( void )  { EviCamera_SendCommand( CamWBOutdoor );  }
void EviCamera_WBOnePushM( void ) { EviCamera_SendCommand( CamWBOnePushM ); }
void EviCamera_WBOnePushT( void ) { EviCamera_SendCommand( CamWBOnePushT ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_AEFullAuto( void ) { EviCamera_SendCommand( CamAEFullAuto ); }
void EviCamera_AEManual( void )   { EviCamera_SendCommand( CamAEManual );   }
void EviCamera_AEShutter( void )  { EviCamera_SendCommand( CamAEShutter );  }
void EviCamera_AEIris( void )     { EviCamera_SendCommand( CamAEIris );     }
void EviCamera_AEBright( void )   { EviCamera_SendCommand( CamAEBright );   }

// ----------------------------------------------------------------------------------------------

void EviCamera_BrightReset( void ) { EviCamera_SendCommand( CamBrightReset ); }
void EviCamera_BrightUp( void )    { EviCamera_SendCommand( CamBrightUp );    }
void EviCamera_BrightDown( void )  { EviCamera_SendCommand( CamBrightDown );  }

// ----------------------------------------------------------------------------------------------

void EviCamera_ShutterReset( void ) { EviCamera_SendCommand( CamShutterReset ); }
void EviCamera_ShutterUp( void )    { EviCamera_SendCommand( CamShutterUp );    }
void EviCamera_ShutterDown( void )  { EviCamera_SendCommand( CamShutterDown );  }

void EviCamera_ShutterDirect( ush ShutterPos, int wait )
{
  EviCamera_SplitShort( (uch*) &ShutterPos, &CamShutterDirect[6] );
  EviCamera_SendCommand( CamShutterDirect );
  if (wait) while(EviCamera_ShutterPosInq() != ShutterPos );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_IrisReset( void ) { EviCamera_SendCommand( CamIrisReset ); }
void EviCamera_IrisUp( void )    { EviCamera_SendCommand( CamIrisUp );    }
void EviCamera_IrisDown( void )  { EviCamera_SendCommand( CamIrisDown );  }

void EviCamera_IrisDirect( ush IrisPos, int wait )
{
  EviCamera_SplitShort( (uch*) &IrisPos, &CamIrisDirect[6] );
  EviCamera_SendCommand( CamIrisDirect );
  if (wait) while(EviCamera_IrisPosInq() != IrisPos );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_GainReset( void ) { EviCamera_SendCommand( CamGainReset ); }
void EviCamera_GainUp( void )    { EviCamera_SendCommand( CamGainUp );    }
void EviCamera_GainDown( void )  { EviCamera_SendCommand( CamGainDown );  }

void EviCamera_GainDirect( ush GainPos, int wait )
{
  EviCamera_SplitShort( (uch*) &GainPos, &CamGainDirect[6] );
  EviCamera_SendCommand( CamGainDirect );
  if (wait) while(EviCamera_GainPosInq() != GainPos );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_BackLightOn( void )     { EviCamera_SendCommand( CamBackLightOn );  }
void EviCamera_BackLightOff( void )    { EviCamera_SendCommand( CamBackLightOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_Preset( uch Pos, uch* Msg )
{
  Msg[7] = Pos;
  EviCamera_SendCommand( Msg );
}

void EviCamera_PresetReset( uch Pos )  { EviCamera_Preset( Pos, CamPresetReset);  }
void EviCamera_PresetSet( uch Pos)     { EviCamera_Preset( Pos, CamPresetSet);    }
void EviCamera_PresetRecall( uch Pos)  { EviCamera_Preset( Pos, CamPresetRecall); }

// ----------------------------------------------------------------------------------------------

void EviCamera_KeyLockOff( void )      { EviCamera_SendCommand( CamKeyLockOff ); }
void EviCamera_KeyLockOn( void )       { EviCamera_SendCommand( CamKeyLockOn );  }

// ----------------------------------------------------------------------------------------------

void EviCamera_IRReceiveOn( void )     { EviCamera_SendCommand( CamIRReceiveOn );    }
void EviCamera_IRReceiveOff( void )    { EviCamera_SendCommand( CamIRReceiveOff );   }
void EviCamera_IRReceiveOnOff( void )  { EviCamera_SendCommand( CamIRReceiveOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_IRRecRetOn( void )      { EviCamera_SendCommand( CamIRRecRetOn );    }
void EviCamera_IRRecRetOff( void )     { EviCamera_SendCommand( CamIRRecRetOff );   }

// ----------------------------------------------------------------------------------------------

void EviCamera_WideConLensSet( uch Lens )
{
  CamWideConLensSet[7] = Lens;
  EviCamera_SendCommand( CamWideConLensSet );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_CommandCancel( uch Socket )
{
  CamCommandCancel[3] = 0x20 | Socket;
  EviCamera_SendCommand( CamCommandCancel );
}

// ----------------------------------------------------------------------------------------------

void EviCamera_PTD1( uch *Msg, uch PanSpeed, uch TiltSpeed )
{
  Msg[6]= PanSpeed;
  Msg[7]= TiltSpeed;
  EviCamera_SendCommand( Msg );
}


// ----------------------------------------------------------------------------------------------

void EviCamera_PTDUp( uch PS, uch TS )        { EviCamera_PTD1(CamPTDUp, PS, TS); }
void EviCamera_PTDDown( uch PS, uch TS )      { EviCamera_PTD1(CamPTDDown, PS, TS); }
void EviCamera_PTDLeft( uch PS, uch TS )      { EviCamera_PTD1( CamPTDLeft, PS, TS); }
void EviCamera_PTDRight( uch PS, uch TS )     { EviCamera_PTD1(CamPTDRight, PS, TS); }
void EviCamera_PTDUpLeft( uch PS, uch TS )    { EviCamera_PTD1(CamPTDUpLeft, PS, TS); }
void EviCamera_PTDUpRight( uch PS, uch TS )   { EviCamera_PTD1( CamPTDUpRight, PS, TS); }
void EviCamera_PTDDownLeft( uch PS, uch TS )  { EviCamera_PTD1( CamPTDDownLeft, PS, TS); }
void EviCamera_PTDDownRight( uch PS, uch TS ) { EviCamera_PTD1( CamPTDDownRight, PS, TS); }

void EviCamera_PTDStop( void )                { EviCamera_SendCommand( CamPTDStop ); }

void EviCamera_PTDAbsPos( uch PS, uch TS, ssh PAP, ssh TAP )
{
  CamPTDAbsPos[6] = PS;
  CamPTDAbsPos[7] = TS;
  EviCamera_SplitShort( (uch*) &PAP, &CamPTDAbsPos[8] );
  EviCamera_SplitShort( (uch*) &TAP, &CamPTDAbsPos[12] );
  EviCamera_SendCommand( CamPTDAbsPos );
} 

void EviCamera_PTDRelPos( uch PS, uch TS, ssh PRP, ssh TRP )
{
  CamPTDRelPos[6] = PS;
  CamPTDRelPos[7] = TS;
  EviCamera_SplitShort( (uch*) &PRP, &CamPTDRelPos[8] );
  EviCamera_SplitShort( (uch*) &TRP, &CamPTDRelPos[12] );
  EviCamera_SendCommand( CamPTDRelPos );
} 

void EviCamera_PTDLimitSet( uch C, ssh P, ssh T )
{
  CamPTDLimitSet[7] = C;
  EviCamera_SplitShort( (uch*) &P,  &CamPTDLimitSet[8] );
  EviCamera_SplitShort( (uch*) &T, &CamPTDLimitSet[12] );
  EviCamera_SendCommand( CamPTDLimitSet );
}

void EviCamera_PTDLimitClear( uch C )
{
  CamPTDLimitClear[7] = C;
  EviCamera_SendCommand( CamPTDLimitClear );
}

void EviCamera_PTDHome( void )         { EviCamera_SendCommand( CamPTDHome );  }
void EviCamera_PTDReset( void )        { EviCamera_SendCommand( CamPTDReset ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_DatascreenOn( void )    { EviCamera_SendCommand( CamDatascreenOn );    }
void EviCamera_DatascreenOff( void )   { EviCamera_SendCommand( CamDatascreenOff );   }
void EviCamera_DatascreenOnOff( void ) { EviCamera_SendCommand( CamDatascreenOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATModeOn( void )    { EviCamera_SendCommand( CamATModeOn );    }
void EviCamera_ATModeOff( void )   { EviCamera_SendCommand( CamATModeOff );   }
void EviCamera_ATModeOnOff( void ) { EviCamera_SendCommand( CamATModeOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATAEOn( void )    { EviCamera_SendCommand( CamATAEOn );    }
void EviCamera_ATAEOff( void )   { EviCamera_SendCommand( CamATAEOff );   }
void EviCamera_ATAEOnOff( void ) { EviCamera_SendCommand( CamATAEOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATAutoZoomOn( void )    { EviCamera_SendCommand( CamATAutoZoomOn );    }
void EviCamera_ATAutoZoomOff( void )   { EviCamera_SendCommand( CamATAutoZoomOff );   }
void EviCamera_ATAutoZoomOnOff( void ) { EviCamera_SendCommand( CamATAutoZoomOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATMDFrameDispOn( void )    { EviCamera_SendCommand( CamATMDFrameDispOn );    }
void EviCamera_ATMDFrameDispOff( void )   { EviCamera_SendCommand( CamATMDFrameDispOff );   }
void EviCamera_ATMDFrameDispOnOff( void ) { EviCamera_SendCommand( CamATMDFrameDispOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATOffsetOn( void )    { EviCamera_SendCommand( CamATOffsetOn );    }
void EviCamera_ATOffsetOff( void )   { EviCamera_SendCommand( CamATOffsetOff );   }
void EviCamera_ATOffsetOnOff( void ) { EviCamera_SendCommand( CamATOffsetOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATAEStartStop( void ) { EviCamera_SendCommand( CamATAEStartStop ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATChase1( void )      { EviCamera_SendCommand( CamATChase1 ); }
void EviCamera_ATChase2( void )      { EviCamera_SendCommand( CamATChase2 ); }
void EviCamera_ATChase3( void )      { EviCamera_SendCommand( CamATChase3 ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATEntry1( void )      { EviCamera_SendCommand( CamATEntry1 ); }
void EviCamera_ATEntry2( void )      { EviCamera_SendCommand( CamATEntry2 ); }
void EviCamera_ATEntry3( void )      { EviCamera_SendCommand( CamATEntry3 ); }
void EviCamera_ATEntry4( void )      { EviCamera_SendCommand( CamATEntry4 ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_MDModeOn( void )    { EviCamera_SendCommand( CamMDModeOn );    }
void EviCamera_MDModeOff( void )   { EviCamera_SendCommand( CamMDModeOff );   }
void EviCamera_MDModeOnOff( void ) { EviCamera_SendCommand( CamMDModeOnOff ); }

// ----------------------------------------------------------------------------------------------

void EviCamera_MDFrame( void )     { EviCamera_SendCommand( CamMDFrame );     }

// ----------------------------------------------------------------------------------------------

void EviCamera_MDDetect( void )    { EviCamera_SendCommand( CamMDDetect );    }

// ----------------------------------------------------------------------------------------------

void EviCamera_ATLostInfo( void )  { EviCamera_SendCommand( CamATLostInfo );  }

// ----------------------------------------------------------------------------------------------

void EviCamera_MDLostInfo( void )  { EviCamera_SendCommand( CamMDLostInfo );  }

// ----------------------------------------------------------------------------------------------

void EviCamera_MDAdjustY( uch Val )
{
  CamMDAdjustY[7] = Val;
  EviCamera_SendCommand( CamMDAdjustY );
}

void EviCamera_MDAdjustH( uch Val )
{
  CamMDAdjustH[7] = Val;
  EviCamera_SendCommand( CamMDAdjustH );
}

void EviCamera_MDAdjustS( uch Val )
{
  CamMDAdjustS[7] = Val;
  EviCamera_SendCommand( CamMDAdjustS );
}

void EviCamera_MDAdjustD( uch Val )
{
  CamMDAdjustD[7] = Val;
  EviCamera_SendCommand( CamMDAdjustD );
}

void EviCamera_MDRef1( void ) { EviCamera_SendCommand( CamMDRef1 );  }
void EviCamera_MDRef2( void ) { EviCamera_SendCommand( CamMDRef2 );  }
void EviCamera_MDRef3( void ) { EviCamera_SendCommand( CamMDRef3 );  }

void EviCamera_MDRefTime( uch Val )
{
  CamMDRefTime[7] = Val;
  EviCamera_SendCommand( CamMDRefTime );
}




// ----------------- read_msg reads Visca messages from the serial port buffer ------------------

int EviCamera_read_msg(int fd1, unsigned char *msg, int len)
{
  int rc, acc = 0, flag = TRUE;

  while (flag)                      // Read complete Visca command character
                                    // by character up to 0xFF which can appear
    {                               // only at the end !
      rc = read(fd1, &msg[acc], 1);

      if (rc > 0)
	 {
           if (msg[acc++] == 0xFF) flag = FALSE;
	 }

      else if (rc < 0)                      // if read error occured
              {
                acc  = 0;
                flag = FALSE;
                fprintf(stderr,"Error reading from link\n");
              }
    }

#ifdef DEBUG
       fprintf(stderr, "\nRX message: ");
       EviCamera_display_hex(msg, acc);
#endif

  return acc;
}


// --- The End ---
