/*
 *    SONY EVI D30/D31 Interface Code, Version 1.3
 *    Copyright (C) Vladimir Kravtchenko 1998
 *    All rights reserved
 *
 *
 *    Permission to use, copy, and distribute this software is hereby
 *    granted without fee.
 *
 *    Redistribution and use of this code, with or without
 *    modification, is subject to the following conditions:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *    3. All advertising materials mentioning features or use of this code
 *       must display the following acknowledgement: This product includes
 *       software developed by Vladimir Kravtchenko <vk@cs.ubc.ca>.
 *    4. The name of the author may not be used to endorse or promote products 
 *       derived from this software without specific prior written permission.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 *    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *    DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *    INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *    STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *    POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * The inclosed code provides an interface to a single SONY EVI D30/D31 Pan/Tilt camera
 * (address 1), connected to computer by a RS232 port. It defines a class (EviCamera)
 * and provides execution of all VISCA commands and inquiries through the class
 * member functions. See class definition and VISCA/RS232C Command List for details.
 *
 *   Before compilation !
 *
 * - define one of the PLATFORM_OS constants, SUN_UNIX or PC_LINUX
 * - define SERIAL_DEVICE, the port to which EVI Camera is connected
 * - define DEBUG if you want the TX/RX messages to be displayed
 * 
 *   Send your comments and questions to vk@cs.ubc.ca
 */

#if !defined(_EVI_CAMERA_H)
#define _EVI_CAMERA_H

#include <stdio.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>

/* #define SUN_UNIX */
/* #define SERIAL_DEVICE "/dev/ttya" */

#define PC_LINUX 
/*#define SERIAL_DEVICE "/dev/ttyS1"*/
#define SERIAL_DEVICE "/dev/ttyR6"

/* #define SERIAL_DEVICE "/dev/cua0" */

/*#define DEBUG*/

// ----------------------------------------------------------------------------------------------


// --- Type definitions ---

typedef unsigned char      uch;
typedef unsigned short     ush;
typedef          short     ssh;
typedef /*unsigned*/ char      Boolean;

// --- Line check flag ---

#define RTS	0x004
#define CTS	0x020
#define DSR	0x100

#define RTS2	0x006

#ifdef SUN_UNIX
       #define LINE_CHECK_FLAG (CTS | RTS)
#endif

#ifdef PC_LINUX
//       #define LINE_CHECK_FLAG (CTS | DSR | RTS)
       #define LINE_CHECK_FLAG (RTS2)
#endif

// --- Misc. constants ---
#if !defined(FALSE)
#define FALSE              0
#endif
#if !defined(TRUE)
#define TRUE               1
#endif

#define VISCA_PACKET_MAX   16                // Max packet length 16 byte

// --- Command constants --- Hex             Dec

#define ZOOM_SPD_MIN       0x02              // 2
#define ZOOM_SPD_MAX       0x07              // 7
#define ZOOM_POS_MIN       0x0000            // 0
#define ZOOM_POS_MAX       0x03FF            // 1023
#define FOCUS_POS_MIN      0x0000            // 0
#define FOCUS_POS_MAX      0xFFFF            // 65535
#define FOCUS_NEAR1        0xCA9F            // 51871
#define FOCUS_NEAR2        0x00A0            // 10
#define FOCUS_FAR          0x10              // 16

#define SHUTTER_POS_MIN    0x0000            // 0
#define SHUTTER_POS_MAX    0x001B            // 27

#define IRIS_POS_MIN       0x0000            // 
#define IRIS_POS_MAX       0x0011            // 17

#define GAIN_POS_MIN       0x0000            // 0
#define GAIN_POS_MAX       0x0007            // 7

#define PRESET_POS_MIN     0x00              // 0
#define PRESET_POS_MAX     0x05              // 5

#define CON_LENS_MIN       0x00              // 0
#define CON_LENS_MAX       0x07              // 7

#define MD_ADJUST_MAX      0x0F              // 15

#define PAN_SPD_MIN        0x01              // 1
#define PAN_SPD_MAX        0x18              // 24
#define TILT_SPD_MIN       0x01              // 1
#define TILT_SPD_MAX       0x14              // 20
                                             //                    By spec (oposite!)
#define UPRIGHT            0x01              //1                   0         
#define DOWNLEFT           0x00              //0                   1

// Measured                 Dec              Hex                   By spec 

#define PAN_ABS_POS_MIN    -860              // 0xFCA4            -880 = 0xFC90    
#define PAN_ABS_POS_MAX     862              // 0x035E             880 = 0x0370   
#define TILT_ABS_POS_MIN   -281              // 0xFEE7            -300 = 0xFED4
#define TILT_ABS_POS_MAX    283              // 0x011B             300 = 0x012C

#define PAN_REL_POS_MIN   -1722              //                  -1760
#define PAN_REL_POS_MAX    1722              //                   1760
#define TILT_REL_POS_MIN   -564              //                   -600
#define TILT_REL_POS_MAX    564              //                    600

// --- Inquiry constants ---

#define POWER_ON           0x02
#define POWER_OFF          0x03

#define FOCUS_AUTO         0x02
#define FOCUS_MANUAL       0x03

#define WB_AUTO            0x00
#define WB_INDOOR          0x01
#define WB_OUTDOOR         0x02
#define WB_ONEPUSH         0x03

#define AE_AUTO            0x00
#define AE_MANUAL          0x03
#define AE_SHUTTER         0x0A
#define AE_IRIS            0x0B
#define AE_BRIGHT          0x0D

#define BACKLIGHT_ON       0x02
#define BACKLIGHT_OFF      0x03

#define KEY_LOCK_OFF       0x00
#define KEY_LOCK_ON        0x02

#define IR_REC_ON          0x02
#define IR_REC_OFF         0x03

#define NTSC               0x00
#define PAL                0x01

#define DATASCREEN_ON      0x02
#define DATASCREEN_OFF     0x03

#define ATMD_MODE_NORM     0x00
#define ATMD_MODE_AT       0x01
#define ATMD_MODE_MD       0x02

#define AT_ENTRY1          0x00
#define AT_ENTRY2          0x01
#define AT_ENTRY3          0x02
#define AT_ENTRY4          0x03

#define MD_REF1            0x00
#define MD_REF2            0x01
#define MD_REF3            0x02

#define AT_MODE_SET        0x00
#define AT_MODE_WORK       0x01
#define AT_MODE_LOST       0x02
#define AT_MODE_MEM        0x03

#define MD_MODE_SET        0x00
#define MD_MODE_UND        0x01
#define MD_MODE_DET        0x02
#define MD_MODE_MEM        0x03

// --- VISCA Commands ---

// Format: { length of Visca command/inquiry in bytes, # of response commands, Visca com/inq itself }

static uch
 
    CamPowerOn[]     = { 6, 2, 0x81, 0x01, 0x04, 0x00, 0x02, 0xFF },
    CamPowerOff[]    = { 6, 2, 0x81, 0x01, 0x04, 0x00, 0x03, 0xFF },

    CamZoomStop[]    = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x00, 0xFF },
    CamZoomTeleS[]   = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x02, 0xFF },
    CamZoomWideS[]   = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x03, 0xFF },
    CamZoomTeleV[]   = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x00, 0xFF },
    CamZoomWideV[]   = { 6, 2, 0x81, 0x01, 0x04, 0x07, 0x00, 0xFF },
    CamZoomDirect[]  = { 9, 2, 0x81, 0x01, 0x04, 0x47, 0x00, 0x00, 0x00, 0x00, 0xFF },  // No Completion ?!

    CamFocusStop[]       = { 6, 2, 0x81, 0x01, 0x04, 0x08, 0x00, 0xFF },
    CamFocusFar[]        = { 6, 2, 0x81, 0x01, 0x04, 0x08, 0x02, 0xFF },
    CamFocusNear[]       = { 6, 2, 0x81, 0x01, 0x04, 0x08, 0x03, 0xFF },
    CamFocusAutoOn[]     = { 6, 2, 0x81, 0x01, 0x04, 0x38, 0x02, 0xFF },
    CamFocusManualOn[]   = { 6, 2, 0x81, 0x01, 0x04, 0x38, 0x03, 0xFF },
    CamFocusAutoManual[] = { 6, 2, 0x81, 0x01, 0x04, 0x38, 0x10, 0xFF },
    CamFocusDirect[]     = { 9, 2, 0x81, 0x01, 0x04, 0x48, 0x00, 0x00, 0x00, 0x00, 0xFF },

    CamWBAuto[]     = { 6, 2, 0x81, 0x01, 0x04, 0x35, 0x00, 0xFF },
    CamWBIndoor[]   = { 6, 2, 0x81, 0x01, 0x04, 0x35, 0x01, 0xFF },
    CamWBOutdoor[]  = { 6, 2, 0x81, 0x01, 0x04, 0x35, 0x02, 0xFF },
    CamWBOnePushM[] = { 6, 2, 0x81, 0x01, 0x04, 0x35, 0x03, 0xFF },
    CamWBOnePushT[] = { 6, 2, 0x81, 0x01, 0x04, 0x10, 0x05, 0xFF },

    CamAEFullAuto[] = { 6, 2, 0x81, 0x01, 0x04, 0x39, 0x00, 0xFF },
    CamAEManual[]   = { 6, 2, 0x81, 0x01, 0x04, 0x39, 0x03, 0xFF },
    CamAEShutter[]  = { 6, 2, 0x81, 0x01, 0x04, 0x39, 0x0A, 0xFF },
    CamAEIris[]     = { 6, 2, 0x81, 0x01, 0x04, 0x39, 0x0B, 0xFF },
    CamAEBright[]   = { 6, 2, 0x81, 0x01, 0x04, 0x39, 0x0D, 0xFF },

    CamBrightReset[] = { 6, 2, 0x81, 0x01, 0x04, 0x0D, 0x00, 0xFF },
    CamBrightUp[]    = { 6, 2, 0x81, 0x01, 0x04, 0x0D, 0x02, 0xFF },
    CamBrightDown[]  = { 6, 2, 0x81, 0x01, 0x04, 0x0D, 0x03, 0xFF },

    CamShutterReset[]  = { 6, 2, 0x81, 0x01, 0x04, 0x0A, 0x00, 0xFF },
    CamShutterUp[]     = { 6, 2, 0x81, 0x01, 0x04, 0x0A, 0x02, 0xFF },
    CamShutterDown[]   = { 6, 2, 0x81, 0x01, 0x04, 0x0A, 0x03, 0xFF },
    CamShutterDirect[] = { 9, 2, 0x81, 0x01, 0x04, 0x4A, 0x00, 0x00, 0x00, 0x00, 0xFF },

    CamIrisReset[]  = { 6, 2, 0x81, 0x01, 0x04, 0x0B, 0x00, 0xFF },
    CamIrisUp[]     = { 6, 2, 0x81, 0x01, 0x04, 0x0B, 0x02, 0xFF },
    CamIrisDown[]   = { 6, 2, 0x81, 0x01, 0x04, 0x0B, 0x03, 0xFF },
    CamIrisDirect[] = { 9, 2, 0x81, 0x01, 0x04, 0x4B, 0x00, 0x00, 0x00, 0x00, 0xFF },

    CamGainReset[]  = { 6, 2, 0x81, 0x01, 0x04, 0x0C, 0x00, 0xFF },
    CamGainUp[]     = { 6, 2, 0x81, 0x01, 0x04, 0x0C, 0x02, 0xFF },
    CamGainDown[]   = { 6, 2, 0x81, 0x01, 0x04, 0x0C, 0x03, 0xFF },
    CamGainDirect[] = { 9, 2, 0x81, 0x01, 0x04, 0x4C, 0x00, 0x00, 0x00, 0x00, 0xFF },

    CamBackLightOn[]  = { 6, 2, 0x81, 0x01, 0x04, 0x33, 0x02, 0xFF },
    CamBackLightOff[] = { 6, 2, 0x81, 0x01, 0x04, 0x33, 0x03, 0xFF },

    CamPresetReset[]  = { 7, 2, 0x81, 0x01, 0x04, 0x3F, 0x00, 0x00, 0xFF },
    CamPresetSet[]    = { 7, 2, 0x81, 0x01, 0x04, 0x3F, 0x01, 0x00, 0xFF },
    CamPresetRecall[] = { 7, 2, 0x81, 0x01, 0x04, 0x3F, 0x02, 0x00, 0xFF },

    CamKeyLockOff[]   = { 6, 2, 0x81, 0x01, 0x04, 0x17, 0x00, 0xFF },
    CamKeyLockOn[]    = { 6, 2, 0x81, 0x01, 0x04, 0x17, 0x02, 0xFF },

    CamIRReceiveOn[]    = { 6, 2, 0x81, 0x01, 0x06, 0x08, 0x02, 0xFF },
    CamIRReceiveOff[]   = { 6, 2, 0x81, 0x01, 0x06, 0x08, 0x03, 0xFF },
    CamIRReceiveOnOff[] = { 6, 2, 0x81, 0x01, 0x06, 0x08, 0x10, 0xFF },

    CamIRRecRetOn[]     = { 8, 2, 0x81, 0x01, 0x7D, 0x01, 0x03, 0x00, 0x00, 0xFF },
    CamIRRecRetOff[]    = { 8, 2, 0x81, 0x01, 0x7D, 0x01, 0x13, 0x00, 0x00, 0xFF },

    CamWideConLensSet[] = { 7, 2, 0x81, 0x01, 0x07, 0x26, 0x00, 0x00, 0xFF },

    CamSetAddressB[]    = { 4, 1, 0x88, 0x30, 0x01, 0xFF },
 // CamSetAddress[]     = { 4, 1, 0x81, 0x30, 0x01, 0xFF },

    CamIFClearB[]       = { 5, 1, 0x88, 0x01, 0x00, 0x01, 0xFF },
 // CamIFClear[]        = { 5, 1, 0x81, 0x01, 0x00, 0x01, 0xFF },

    CamCommandCancel[]  = { 3, 1, 0x81, 0x20, 0xFF },
    
    CamPTDUp[]          = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x01, 0xFF },
    CamPTDDown[]        = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x02, 0xFF },
    CamPTDLeft[]        = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x01, 0x03, 0xFF },
    CamPTDRight[]       = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x02, 0x03, 0xFF },
    CamPTDUpLeft[]      = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x01, 0x01, 0xFF },
    CamPTDUpRight[]     = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x02, 0x01, 0xFF },
    CamPTDDownLeft[]    = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x01, 0x02, 0xFF },
    CamPTDDownRight[]   = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x02, 0x02, 0xFF },
    CamPTDStop[]        = { 9, 2, 0x81, 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x03, 0xFF },
    CamPTDAbsPos[]      = { 15, 2, 0x81, 0x01, 0x06, 0x02, 0x00, 0x00, 0,0,0,0, 0,0,0,0, 0xFF },
    CamPTDRelPos[]      = { 15, 2, 0x81, 0x01, 0x06, 0x03, 0x00, 0x00, 0,0,0,0, 0,0,0,0, 0xFF },
    CamPTDHome[]        = {  5, 2, 0x81, 0x01, 0x06, 0x04, 0xFF },
    CamPTDReset[]       = {  5, 2, 0x81, 0x01, 0x06, 0x05, 0xFF },
    CamPTDLimitSet[]    = { 15, 2, 0x81, 0x01, 0x06, 0x07, 0x00, 0x00, 0,0,0,0, 0,0,0,0, 0xFF },
    CamPTDLimitClear[]  = { 15, 2, 0x81, 0x01, 0x06, 0x07, 0x01, 0x00, 0x7,0x0F,0x0F,0x0F,
                                                                       0x7,0x0F,0x0F,0x0F, 0xFF },

    CamDatascreenOn[]       = { 6, 2, 0x81, 0x01, 0x06, 0x06, 0x02, 0xFF },
    CamDatascreenOff[]      = { 6, 2, 0x81, 0x01, 0x06, 0x06, 0x03, 0xFF },
    CamDatascreenOnOff[]    = { 6, 2, 0x81, 0x01, 0x06, 0x06, 0x10, 0xFF },

    CamATModeOn[]           = { 6, 2, 0x81, 0x01, 0x07, 0x01, 0x02, 0xFF },
    CamATModeOff[]          = { 6, 2, 0x81, 0x01, 0x07, 0x01, 0x03, 0xFF },
    CamATModeOnOff[]        = { 6, 2, 0x81, 0x01, 0x07, 0x01, 0x10, 0xFF },

    CamATAEOn[]             = { 6, 2, 0x81, 0x01, 0x07, 0x02, 0x02, 0xFF },
    CamATAEOff[]            = { 6, 2, 0x81, 0x01, 0x07, 0x02, 0x03, 0xFF },
    CamATAEOnOff[]          = { 6, 2, 0x81, 0x01, 0x07, 0x02, 0x10, 0xFF },

    CamATAutoZoomOn[]       = { 6, 2, 0x81, 0x01, 0x07, 0x03, 0x02, 0xFF },
    CamATAutoZoomOff[]      = { 6, 2, 0x81, 0x01, 0x07, 0x03, 0x03, 0xFF },
    CamATAutoZoomOnOff[]    = { 6, 2, 0x81, 0x01, 0x07, 0x03, 0x10, 0xFF },

    CamATMDFrameDispOn[]    = { 6, 2, 0x81, 0x01, 0x07, 0x04, 0x02, 0xFF },
    CamATMDFrameDispOff[]   = { 6, 2, 0x81, 0x01, 0x07, 0x04, 0x03, 0xFF },
    CamATMDFrameDispOnOff[] = { 6, 2, 0x81, 0x01, 0x07, 0x04, 0x10, 0xFF },

    CamATOffsetOn[]         = { 6, 2, 0x81, 0x01, 0x07, 0x05, 0x02, 0xFF },
    CamATOffsetOff[]        = { 6, 2, 0x81, 0x01, 0x07, 0x05, 0x03, 0xFF },
    CamATOffsetOnOff[]      = { 6, 2, 0x81, 0x01, 0x07, 0x05, 0x10, 0xFF },

    CamATAEStartStop[]      = { 6, 2, 0x81, 0x01, 0x07, 0x06, 0x10, 0xFF },

    CamATChase1[]           = { 6, 2, 0x81, 0x01, 0x07, 0x07, 0x00, 0xFF },
    CamATChase2[]           = { 6, 2, 0x81, 0x01, 0x07, 0x07, 0x01, 0xFF },
    CamATChase3[]           = { 6, 2, 0x81, 0x01, 0x07, 0x07, 0x02, 0xFF },

    CamATEntry1[]           = { 6, 2, 0x81, 0x01, 0x07, 0x15, 0x00, 0xFF },
    CamATEntry2[]           = { 6, 2, 0x81, 0x01, 0x07, 0x15, 0x01, 0xFF },
    CamATEntry3[]           = { 6, 2, 0x81, 0x01, 0x07, 0x15, 0x02, 0xFF },
    CamATEntry4[]           = { 6, 2, 0x81, 0x01, 0x07, 0x15, 0x03, 0xFF },

    CamMDModeOn[]           = { 6, 2, 0x81, 0x01, 0x07, 0x08, 0x02, 0xFF },
    CamMDModeOff[]          = { 6, 2, 0x81, 0x01, 0x07, 0x08, 0x03, 0xFF },
    CamMDModeOnOff[]        = { 6, 2, 0x81, 0x01, 0x07, 0x08, 0x10, 0xFF },

    CamMDFrame[]            = { 5, 2, 0x81, 0x01, 0x07, 0x09, 0xFF },
    CamMDDetect[]           = { 6, 2, 0x81, 0x01, 0x07, 0x0A, 0x10, 0xFF },
    CamATLostInfo[]         = { 7, 2, 0x81, 0x01, 0x06, 0x20, 0x07, 0x20, 0xFF },
    CamMDLostInfo[]         = { 7, 2, 0x81, 0x01, 0x06, 0x20, 0x07, 0x21, 0xFF },

    CamMDAdjustY[]          = { 7, 2, 0x81, 0x01, 0x07, 0x0B, 0x00, 0x00, 0xFF },
    CamMDAdjustH[]          = { 7, 2, 0x81, 0x01, 0x07, 0x0C, 0x00, 0x00, 0xFF },
    CamMDAdjustS[]          = { 7, 2, 0x81, 0x01, 0x07, 0x0D, 0x00, 0x00, 0xFF },
    CamMDAdjustD[]          = { 7, 2, 0x81, 0x01, 0x07, 0x0F, 0x00, 0x00, 0xFF },

    CamMDRef1[]             = { 6, 2, 0x81, 0x01, 0x07, 0x10, 0x00, 0xFF },
    CamMDRef2[]             = { 6, 2, 0x81, 0x01, 0x07, 0x10, 0x01, 0xFF },
    CamMDRef3[]             = { 6, 2, 0x81, 0x01, 0x07, 0x10, 0x02, 0xFF },
    CamMDRefTime[]          = { 7, 2, 0x81, 0x01, 0x07, 0x0B, 0x00, 0x00, 0xFF },

// --- VISCA Inquiries ---

    CamPowerInq[]         = { 5, 1, 0x81, 0x09, 0x04, 0x00, 0xFF },
    CamZoomPosInq[]       = { 5, 1, 0x81, 0x09, 0x04, 0x47, 0xFF },
    CamFocusModeInq[]     = { 5, 1, 0x81, 0x09, 0x04, 0x38, 0xFF },
    CamFocusPosInq[]      = { 5, 1, 0x81, 0x09, 0x04, 0x48, 0xFF },
    CamWBModeInq[]        = { 5, 1, 0x81, 0x09, 0x04, 0x35, 0xFF },
    CamAEModeInq[]        = { 5, 1, 0x81, 0x09, 0x04, 0x39, 0xFF },
    CamShutterPosInq[]    = { 5, 1, 0x81, 0x09, 0x04, 0x4A, 0xFF },
    CamIrisPosInq[]       = { 5, 1, 0x81, 0x09, 0x04, 0x4B, 0xFF },
    CamGainPosInq[]       = { 5, 1, 0x81, 0x09, 0x04, 0x4C, 0xFF },
    CamBackLightModeInq[] = { 5, 1, 0x81, 0x09, 0x04, 0x33, 0xFF },
    CamMemoryInq[]        = { 5, 1, 0x81, 0x09, 0x04, 0x3F, 0xFF },
    CamKeyLockInq[]       = { 5, 1, 0x81, 0x09, 0x04, 0x17, 0xFF },
    CamIDInq[]            = { 5, 1, 0x81, 0x09, 0x04, 0x22, 0xFF },
    CamIRReceiveModeInq[] = { 5, 1, 0x81, 0x09, 0x06, 0x08, 0xFF },
    CamVideoSystemInq[]   = { 5, 1, 0x81, 0x09, 0x06, 0x23, 0xFF },
    CamWideConLensInq[]   = { 5, 1, 0x81, 0x09, 0x07, 0x26, 0xFF },

    CamPanTiltModeInq[]   = { 5, 1, 0x81, 0x09, 0x06, 0x10, 0xFF },
    CamPanTiltMaxSpdInq[] = { 5, 1, 0x81, 0x09, 0x06, 0x11, 0xFF },
    CamPanTiltPosInq[]    = { 5, 1, 0x81, 0x09, 0x06, 0x12, 0xFF },

    CamDataScreenInq[]    = { 5, 1, 0x81, 0x09, 0x06, 0x06, 0xFF },
    CamAT_MDModeInq[]     = { 5, 1, 0x81, 0x09, 0x07, 0x22, 0xFF },
    CamATModeInq[]        = { 5, 1, 0x81, 0x09, 0x07, 0x23, 0xFF },
    CamATEntryInq[]       = { 5, 1, 0x81, 0x09, 0x07, 0x15, 0xFF },
    CamMDModeInq[]        = { 5, 1, 0x81, 0x09, 0x07, 0x24, 0xFF },
    CamATObjPosInq[]      = { 5, 1, 0x81, 0x09, 0x07, 0x20, 0xFF },
    CamMDObjPosInq[]      = { 5, 1, 0x81, 0x09, 0x07, 0x21, 0xFF },
    CamMDYLevelInq[]      = { 5, 1, 0x81, 0x09, 0x07, 0x0B, 0xFF },
    CamMDHueLevelInq[]    = { 5, 1, 0x81, 0x09, 0x07, 0x0C, 0xFF },
    CamMDSizeInq[]        = { 5, 1, 0x81, 0x09, 0x07, 0x0D, 0xFF },
    CamMDDispTimeInq[]    = { 5, 1, 0x81, 0x09, 0x07, 0x0F, 0xFF },
    CamMDRefModeInq[]     = { 5, 1, 0x81, 0x09, 0x07, 0x10, 0xFF },
    CamMDRefTimeInq[]     = { 5, 1, 0x81, 0x09, 0x07, 0x11, 0xFF };

 // CamIRRecRetInq[]      = { 5, 1, 0x81, 0x09, 0x07, 0x00, 0xFF };

// ----------------------------------------------------------------------------------------------


void EviCamera_CreateRS232Link( void );
void EviCamera_CloseRS232Link( void );
void EviCamera_InitializeCamera( void );

// --- VISCA Commands ---

void EviCamera_PowerOn( void );
void EviCamera_PowerOff( void );

void EviCamera_ZoomStop( void );
void EviCamera_ZoomTeleS( int wait );   // wait = TRUE  means wait till the end of execution
void EviCamera_ZoomWideS( int wait );   // wait = FALSE means do not wait, proceed asap

void EviCamera_ZoomTeleV( uch speed, int wait );
void EviCamera_ZoomWideV( uch speed, int wait );
void EviCamera_ZoomDirect( ush ZoomPos, int wait );

void EviCamera_FocusStop( void );
void EviCamera_FocusFar( int wait );
void EviCamera_FocusNear( int wait );
void EviCamera_FocusAutoOn( void );
void EviCamera_FocusManualOn( void );
void EviCamera_FocusAutoManual( void );
void EviCamera_FocusDirect( ush FocusPos, int wait );

void EviCamera_WBAuto( void );
void EviCamera_WBIndoor( void );
void EviCamera_WBOutdoor( void );
void EviCamera_WBOnePushM( void );
void EviCamera_WBOnePushT( void );

void EviCamera_AEFullAuto( void );
void EviCamera_AEManual( void );
void EviCamera_AEShutter( void );
void EviCamera_AEIris( void );
void EviCamera_AEBright( void );

void EviCamera_BrightReset( void );
void EviCamera_BrightUp( void );
void EviCamera_BrightDown( void );

void EviCamera_ShutterReset( void );
void EviCamera_ShutterUp( void );
void EviCamera_ShutterDown( void );
void EviCamera_ShutterDirect( ush ShutterPos, int wait );

void EviCamera_IrisReset( void );
void EviCamera_IrisUp( void );
void EviCamera_IrisDown( void );
void EviCamera_IrisDirect( ush IrisPos, int wait );

void EviCamera_GainReset( void );
void EviCamera_GainUp( void );
void EviCamera_GainDown( void );
void EviCamera_GainDirect( ush GainPos, int wait );

void EviCamera_BackLightOn( void );
void EviCamera_BackLightOff( void );

void EviCamera_PresetReset( uch Pos );
void EviCamera_PresetSet( uch Pos );
void EviCamera_PresetRecall( uch Pos );

void EviCamera_KeyLockOff( void );
void EviCamera_KeyLockOn( void );

void EviCamera_IRReceiveOn( void );
void EviCamera_IRReceiveOff( void );
void EviCamera_IRReceiveOnOff( void );

void EviCamera_IRRecRetOn( void );
void EviCamera_IRRecRetOff( void );

void EviCamera_WideConLensSet( uch Lens );

void EviCamera_CommandCancel( uch Socket );

void EviCamera_PTDUp( uch PS, uch TS );
void EviCamera_PTDDown( uch PS, uch TS );
void EviCamera_PTDLeft( uch PS, uch TS );
void EviCamera_PTDRight( uch PS, uch TS );
void EviCamera_PTDUpLeft( uch PS, uch TS );
void EviCamera_PTDUpRight( uch PS, uch TS );
void EviCamera_PTDDownLeft( uch PS, uch TS );
void EviCamera_PTDDownRight( uch PS, uch TS );
void EviCamera_PTDStop2( void );
void EviCamera_PTDStop( void );
void EviCamera_PTDAbsPos( uch PS, uch TS, ssh PRP, ssh TRP );
void EviCamera_PTDRelPos( uch PS, uch TS, ssh PRP, ssh TRP );
void EviCamera_PTDHome( void );
void EviCamera_PTDReset( void );
void EviCamera_PTDLimitSet( uch C, ssh P, ssh T );
void EviCamera_PTDLimitClear( uch C );

void EviCamera_DatascreenOn( void );
void EviCamera_DatascreenOff( void );
void EviCamera_DatascreenOnOff( void );

void EviCamera_ATModeOn( void );
void EviCamera_ATModeOff( void );
void EviCamera_ATModeOnOff( void );

void EviCamera_ATAEOn( void );
void EviCamera_ATAEOff( void );
void EviCamera_ATAEOnOff( void );

void EviCamera_ATAutoZoomOn( void );
void EviCamera_ATAutoZoomOff( void );
void EviCamera_ATAutoZoomOnOff( void );

void EviCamera_ATMDFrameDispOn( void );
void EviCamera_ATMDFrameDispOff( void );
void EviCamera_ATMDFrameDispOnOff( void );

void EviCamera_ATOffsetOn( void );
void EviCamera_ATOffsetOff( void );
void EviCamera_ATOffsetOnOff( void );

void EviCamera_ATAEStartStop( void );

void EviCamera_ATChase1( void );
void EviCamera_ATChase2( void );
void EviCamera_ATChase3( void );

void EviCamera_ATEntry1( void );
void EviCamera_ATEntry2( void );
void EviCamera_ATEntry3( void );
void EviCamera_ATEntry4( void );

void EviCamera_MDModeOn( void );
void EviCamera_MDModeOff( void );
void EviCamera_MDModeOnOff( void );

void EviCamera_MDFrame( void );

void EviCamera_MDDetect( void );
void EviCamera_ATLostInfo( void );
void EviCamera_MDLostInfo( void );

void EviCamera_MDAdjustY( uch Val );
void EviCamera_MDAdjustH( uch Val );
void EviCamera_MDAdjustS( uch Val );
void EviCamera_MDAdjustD( uch Val );
void EviCamera_MDRef1( void );
void EviCamera_MDRef2( void );
void EviCamera_MDRef3( void );
void EviCamera_MDRefTime( uch Val );

// --- VISCA Inquiries ---

uch EviCamera_PowerInq( void );
ush EviCamera_ZoomPosInq( void );
uch EviCamera_FocusModeInq( void );
ush EviCamera_FocusPosInq( void );
uch EviCamera_WBModeInq( void );
uch EviCamera_AEModeInq( void );
ush EviCamera_ShutterPosInq( void );
ush EviCamera_IrisPosInq( void );
ush EviCamera_GainPosInq( void );
uch EviCamera_BackLightModeInq( void );
uch EviCamera_MemoryInq( void );
uch EviCamera_KeyLockInq( void );
uch EviCamera_IDInq( void );
uch EviCamera_IRReceiveModeInq( void );
uch EviCamera_VideoSystemInq( void );
uch EviCamera_WideConLensInq( void );

ush EviCamera_PanTiltModeInq( void );
void EviCamera_PanTiltMaxSpdInq( uch *PanSpd, uch *TiltSpd );
void EviCamera_PanTiltPosInq( ssh *PanPos, ssh *TiltPos );

uch EviCamera_DataScreenInq( void );
uch EviCamera_AT_MDModeInq( void );
ush EviCamera_ATModeInq( void );
uch EviCamera_ATEntryInq( void );
ush EviCamera_MDModeInq( void );
void EviCamera_ATObjPosInq( uch *X, uch *Y, uch *status );
void EviCamera_MDObjPosInq( uch *X, uch *Y, uch *status );
uch EviCamera_MDYLevelInq( void );
uch EviCamera_MDHueLevelInq( void );
uch EviCamera_MDSizeInq( void );
uch EviCamera_MDDispTimeInq( void );
uch EviCamera_MDRefModeInq( void );
uch EviCamera_MDRefTimeInq( void );
/*
            uch IRRecRetInq( void );
*/
void EviCamera_PTD1( uch *Msg, uch PanSpeed, uch TiltSpeed );
void EviCamera_SendCommand( uch *msg );
void EviCamera_SendCommandPanTilt( int op, uch panSpd,uch tiltSpd);
void EviCamera_SendCommandZoom( int op,uch speed);
void EviCamera_SendCommand2( uch *msg );

       
void EviCamera_SplitShort( uch *src, uch *dest );
void EviCamera_AsmShort( uch *src, uch *dest );
void EviCamera_WaitReady( int NoCom );
void EviCamera_Preset( uch Pos, uch *Msg );


Boolean EviCamera_set_attr( int fd );
Boolean EviCamera_create_link( int *fd1 );
void    EviCamera_close_link( int *fd1 );
void    EviCamera_display_hex( unsigned char *msg, int len );
int     EviCamera_read_msg( int fd1, unsigned char *msg, int len);
Boolean EviCamera_write_msg( int fd1, unsigned char *msg, unsigned char len );


/* Autres fonctions rajoutees pour simplifier les appels */
int EVI_Init();
int EVI_ControlParam(int width,int height,int options,double *fuzzySet);
int EVI_changeSpeed(int panSpd,int tiltSpd,int zoomSpd);
void EVI_GoHome();
void EVI_Stop();
void EVI_Close();
int EVI_GoTo(float pan,float tilt,float zoom);
int EVI_GoToD70(float pan,float tilt,float zoom, int zoomEnable);
 
int EVI_GetParam(int *pan,int *tilt,int *zoom);
int EVI_GetParamD70(float *pan,float *tilt,float *zoom);
int EVI_Control(int *boundBox);
int EVI_CenterBox(int *box);
int EVI_ChangeProperty(int prop,int val);
int EVI_fuzzyControl(int *box);


#endif


