/***************************************************************************
 *            includes_rfid.h
 *
 *  Thu Jul 17 11:03:35 2008
 *  Made by Antoine Roguez & Leo Bernard
 *  aroguez@laas.fr leo.bernard@laas.fr
 ****************************************************************************/

#ifndef __ANTRFIDLIB_H
#define __ANTRFIDLIB_H

#include <time.h>

#define NBANT 8


/*
        Error codes
*/
typedef enum {
	CAENRFID_StatusOK		 =  0,  /* Operation completed successfully */
	CAENRFID_PortError		 = -2,  /* Error on selected port*/
	CAENRFID_InvalidHandleError	 = -5,   /* Invalid Handle*/
	CAENRFID_OutOfMemoryError	 = -6,
	CAENRFID_CommunicationError	 = -7,
	CAENRFID_GenericError		 = -10, /* Generic error*/
	CAENRFID_InvalidParam		 = -11,  /* Invalid parameter error*/
	CAENRFID_NotConnected		= -12, /*No client connected*/
	CAENRFID_TagNotFound		=-15  /*Tag asked not found*/
} CAENRFIDErrorCodes;

/*
 		Protocols
 */
typedef enum {
	ISO18000_6B   = 0,
	EPC_C1G1      = 1,
	EM	      = 2,
	EPC_C1G2      = 3,
	MULTIPROTOCOL = 4,
	EPC119	      = 5
} CAENRFIDProtocol;



/*
 list of the detected tags 
 */
struct internalTag{
	char Tag_ID[16];/* Tag's ID */
	char *Tag_MEMORY;/* Tag's memory */
	int last_detec[NBANT]; /* result of last detection */
};
typedef struct internalTag internalTag;


typedef struct internalList{
	struct internalTag *Tag;
	struct internalList *next; 
}internalList;


/*
 list of all the information about the tags
 */
struct externalTag{
	char Tag_ID[16];/* Tag's ID */
	char *Tag_MEMORY;/* Tag's memory */
	double angle; /* angle of the tag in the robot repere */
	double ecartTypeAngle; /* ecart type de l'angle */
	double longueur; /* length of the tag in the robot repere */
	double ecartTypeLongueur; /* ecart type de la longueur */
};
typedef struct externalTag externalTag;

typedef struct tagList{
	struct externalTag *Tag;
	struct tagList *next; 
}tagList;



/*
        RS232_connect
        -----------------------------------------------------------------------------
        Parameters:
            [in] port_num	: The COM port on which the reader is connected number of Windows
								(eg. 1 for COM1)
	    [in] proto		: The wanted protocol
								(see CAENRFIDProtocol enum)
            [in]usb2serial	: 1 if using an usb2serial adapter, 0 else.
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The function connects a reader to the computer on the wanted COM port
*/
CAENRFIDErrorCodes RS232_connect(int port_num, CAENRFIDProtocol proto,int usb2serial);


/*
        RS232_disconnect
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The function disconnects the reader from the computer on the wanted port
			It sets free the whole allocated memory
*/
CAENRFIDErrorCodes RS232_disconnect();


/*
        Activateantenna
        -----------------------------------------------------------------------------
        Parameters:
            [in] antenna	: The antenna wanted to be activated (from 0 to 7)
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The reader will try to connect itself to the wanted antennas. (antenna & antenna+4)
				It won't work if the antenna is covered, even partially
*/
CAENRFIDErrorCodes Activateantenna(int antenna);

/*
        Desactivateantenna
        -----------------------------------------------------------------------------
        Parameters:
            [in] antenna	: The antenna wanted to be desactivated (from 0 to 7)
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The reader will disconnect itself from the wanted antennas. (antenna & antenna+4) 
*/
CAENRFIDErrorCodes Desactivateantenna(int antenna);


/*
        get_power
        -----------------------------------------------------------------------------
        Parameters:
            [out] powerR	: The RF power sent by the reader
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            Permits to read the power settings of the reader
*/
CAENRFIDErrorCodes get_power(unsigned int *powerR);


/*
        set_get_power
        -----------------------------------------------------------------------------
        Parameters:
			[in]  power		: the wanted RF power to be set
            [out] powerR	: The RF power sent by the reader
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            Permits to set the RF power of the reader
*/
CAENRFIDErrorCodes set_get_power(int power,unsigned int *powerR);


/*
        reset_External_list
        -----------------------------------------------------------------------------
        Description:
            resets the external list of tags (externalList)
*/
void reset_External_list();

/*
        reset_Internal_list
        -----------------------------------------------------------------------------
        Description:
            resets the internal list of New tags (internalList)
*/
void reset_Internal_list();



/*
        acquisition
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The real acquisition work. The found tag's IDs are put in an internal list.
			All antennas are asked one after an other.
*/
CAENRFIDErrorCodes acquisition();

/*
        getTagInfo
        -----------------------------------------------------------------------------
        Returns:
            the info of the asked tag.
        -----------------------------------------------------------------------------
        Description:
            Take the specification inthe internal list to determine the angle, and the proba
*/
externalTag *getTagInfo(internalTag *Tag);


/*
		get_Tag_by_pos
        -----------------------------------------------------------------------------
        Parameters:
			[in]  position	: the position in the list of wanted tag
        -----------------------------------------------------------------------------
        Returns:
            The wanted tag
        -----------------------------------------------------------------------------
        Description:
            Permits to get the tag's ID and other information about the Tag stored during the acquisition


*/
internalTag *get_Internal_Tag_by_pos(int position);

/*
		get_Tag_by_Id
        -----------------------------------------------------------------------------
        Parameters:
			[in]  Id	: the Id the wanted tag
        -----------------------------------------------------------------------------
        Returns:
            The wanted tag
        -----------------------------------------------------------------------------
        Description:
         	Permits to get various information about the Tag stored during the acquisition

*/
internalTag *get_Internal_Tag_by_Id(char *Tag_ID);


/*
		init_RFID
        -----------------------------------------------------------------------------
        Parameters:
			[int]  com	: the wanted port com like in windows (1   9)
        -----------------------------------------------------------------------------
        Returns:
            error code
        -----------------------------------------------------------------------------
        Description:
         	allow you to initialyse the connection

*/

CAENRFIDErrorCodes init_RFID(int com);
/*
		close_RFID
        -----------------------------------------------------------------------------
        Parameters:
			nothing
        -----------------------------------------------------------------------------
        Returns:
            error code
        -----------------------------------------------------------------------------
        Description:
         	allow you to deconnect the reader

*/
CAENRFIDErrorCodes close_RFID();

#endif
